@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -36,7 +36,7 @@
 #include <curand_mtgp32_host.h>
 #include <curand_mtgp32dc_p_11213.h>
 
-#define CUPRAND_DEFAULT_MAX_BLOCK_SIZE 256
+#define CURAND_DEFAULT_MAX_BLOCK_SIZE 256
 
 #define CUDA_CALL(x) do { \
     cudaError_t error = (x);\
@@ -63,8 +63,8 @@ size_t next_power2(size_t x)
 
 template<typename GeneratorState>
 __global__
-__launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE)
-void init_kernel(GeneratorState * states,
+__launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_kernel(GeneratorState*          states,
                  const unsigned long long seed,
                  const unsigned long long offset)
 {
@@ -76,12 +76,12 @@ void init_kernel(GeneratorState * states,
 
 template<typename GeneratorState, typename T, typename GenerateFunc, typename Extra>
 __global__
-__launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE)
-void generate_kernel(GeneratorState * states,
-                     T * data,
-                     const size_t size,
-                     GenerateFunc generate_func,
-                     const Extra extra)
+__launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_kernel(GeneratorState* states,
+                     T*              data,
+                     const size_t    size,
+                     GenerateFunc    generate_func,
+                     const Extra     extra)
 {
     const unsigned int state_id = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int stride = gridDim.x * blockDim.x;
@@ -108,12 +108,11 @@ struct runner
            const unsigned long long offset)
     {
         const size_t states_size = blocks * threads;
-        CUDA_CALL(
-            cudaMalloc(reinterpret_cast<void**>(&states), states_size * sizeof(GeneratorState)));
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(GeneratorState)));
 
         init_kernel<<<blocks, threads>>>(states, seed, offset);
 
-        CUDA_CALL(cudaPeekAtLastError());
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
     }
 
@@ -128,12 +127,13 @@ struct runner
     }
 
     template<typename T, typename GenerateFunc, typename Extra>
-    void generate(const size_t blocks,
-                  const size_t threads,
-                  T * data,
-                  const size_t size,
+    void generate(const size_t        blocks,
+                  const size_t        threads,
+                  cudaStream_t        stream,
+                  T*                  data,
+                  const size_t        size,
                   const GenerateFunc& generate_func,
-                  const Extra extra)
+                  const Extra         extra)
     {
         generate_kernel<<<blocks, threads>>>(states, data, size, generate_func, extra);
     }
@@ -141,12 +141,12 @@ struct runner
 
 template<typename T, typename GenerateFunc, typename Extra>
 __global__
-__launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE)
-void generate_kernel(curandStateMtgp32_t * states,
-                     T * data,
-                     const size_t size,
-                     GenerateFunc generate_func,
-                     const Extra extra)
+__launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_kernel(curandStateMtgp32_t* states,
+                     T*                   data,
+                     const size_t         size,
+                     GenerateFunc         generate_func,
+                     const Extra          extra)
 {
     const unsigned int state_id = blockIdx.x;
     const unsigned int thread_id = threadIdx.x;
@@ -160,7 +160,13 @@ void generate_kernel(curandStateMtgp32_t * states,
     __syncthreads();
 
     const size_t r = size%blockDim.x;
-    const size_t size_rounded_up = r == 0 ? size : size + (blockDim.x - r);
+    const size_t size_rounded_down = size - r;
+    const size_t size_rounded_up   = r == 0 ? size : size_rounded_down + blockDim.x;
+    while(index < size_rounded_down)
+    {
+        data[index] = generate_func(&state, extra);
+        index += stride;
+    }
     while(index < size_rounded_up)
     {
         auto value = generate_func(&state, extra);
@@ -187,10 +193,9 @@ struct runner<curandStateMtgp32_t>
            const unsigned long long /* offset */)
     {
         const size_t states_size = std::min((size_t)200, blocks);
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&states),
-                             states_size * sizeof(curandStateMtgp32_t)));
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(curandStateMtgp32_t)));
 
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&d_param), sizeof(mtgp32_kernel_params)));
+        CUDA_CALL(cudaMalloc(&d_param, sizeof(mtgp32_kernel_params)));
         CURAND_CALL(curandMakeMTGP32Constants(mtgp32dc_params_fast_11213, d_param));
         CURAND_CALL(curandMakeMTGP32KernelState(states, mtgp32dc_params_fast_11213, d_param, states_size, seed));
     }
@@ -209,18 +214,23 @@ struct runner<curandStateMtgp32_t>
     template<typename T, typename GenerateFunc, typename Extra>
     void generate(const size_t blocks,
                   const size_t /* threads */,
-                  T * data,
-                  const size_t size,
+                  cudaStream_t        stream,
+                  T*                  data,
+                  const size_t        size,
                   const GenerateFunc& generate_func,
-                  const Extra extra)
+                  const Extra         extra)
     {
-        generate_kernel<<<std::min((size_t)200, blocks), 256>>>(states, data, size, generate_func, extra);
+        generate_kernel<<<std::min((size_t)200, blocks), 256, 0, stream>>>(states,
+                                                                           data,
+                                                                           size,
+                                                                           generate_func,
+                                                                           extra);
     }
 };
 
 template<typename GeneratorState, typename SobolType>
-__global__ __launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE) void init_sobol_kernel(
-    GeneratorState* states, SobolType* directions, SobolType offset)
+__global__ __launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_sobol_kernel(GeneratorState* states, SobolType* directions, SobolType offset)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id = blockIdx.x * blockDim.x + threadIdx.x;
@@ -230,8 +240,11 @@ __global__ __launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE) void init_sobol_ker
 }
 
 template<typename GeneratorState, typename SobolType>
-__global__ __launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE) void init_scrambled_sobol_kernel(
-    GeneratorState* states, SobolType* directions, SobolType* scramble_constants, SobolType offset)
+__global__ __launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void init_scrambled_sobol_kernel(GeneratorState* states,
+                                 SobolType*      directions,
+                                 SobolType*      scramble_constants,
+                                 SobolType       offset)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id  = blockIdx.x * blockDim.x + threadIdx.x;
@@ -245,12 +258,12 @@ __global__ __launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE) void init_scrambled
 
 // generate_kernel for the sobol generators
 template<typename GeneratorState, typename T, typename GenerateFunc, typename Extra>
-__global__ __launch_bounds__(CUPRAND_DEFAULT_MAX_BLOCK_SIZE) void generate_sobol_kernel(
-    GeneratorState* states,
-    T*              data,
-    const size_t    size,
-    GenerateFunc    generate_func,
-    const Extra     extra)
+__global__ __launch_bounds__(CURAND_DEFAULT_MAX_BLOCK_SIZE)
+void generate_sobol_kernel(GeneratorState* states,
+                           T*              data,
+                           const size_t    size,
+                           GenerateFunc    generate_func,
+                           const Extra     extra)
 {
     const unsigned int dimension = blockIdx.y;
     const unsigned int state_id = blockIdx.x * blockDim.x + threadIdx.x;
@@ -284,15 +297,16 @@ struct runner<curandStateSobol32_t>
     {
         this->dimensions = dimensions;
 
+        curandDirectionVectors32_t* h_directions;
+        CURAND_CALL(
+            curandGetDirectionVectors32(&h_directions, CURAND_DIRECTION_VECTORS_32_JOEKUO6));
+
         const size_t states_size = blocks * threads * dimensions;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&states),
-                             states_size * sizeof(curandStateSobol32_t)));
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(curandStateSobol32_t)));
 
-        curandDirectionVectors32_t * h_directions;
-        CURAND_CALL(curandGetDirectionVectors32(&h_directions, CURAND_DIRECTION_VECTORS_32_JOEKUO6));
         unsigned int* directions;
         const size_t  size = dimensions * sizeof(unsigned int) * 32;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&directions), size));
+        CUDA_CALL(cudaMalloc(&directions, size));
         CUDA_CALL(cudaMemcpy(directions, h_directions, size, cudaMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
@@ -301,7 +315,7 @@ struct runner<curandStateSobol32_t>
             directions,
             static_cast<unsigned int>(offset));
 
-        CUDA_CALL(cudaPeekAtLastError());
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
 
         CUDA_CALL(cudaFree(directions));
@@ -318,19 +332,20 @@ struct runner<curandStateSobol32_t>
     }
 
     template<typename T, typename GenerateFunc, typename Extra>
-    void generate(const size_t blocks,
-                  const size_t threads,
-                  T * data,
-                  const size_t size,
+    void generate(const size_t        blocks,
+                  const size_t        threads,
+                  cudaStream_t        stream,
+                  T*                  data,
+                  const size_t        size,
                   const GenerateFunc& generate_func,
-                  const Extra extra)
+                  const Extra         extra)
     {
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
-        generate_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(states,
-                                                                       data,
-                                                                       size / dimensions,
-                                                                       generate_func,
-                                                                       extra);
+        generate_sobol_kernel<<<dim3(blocks_x, dimensions), threads, 0, stream>>>(states,
+                                                                                  data,
+                                                                                  size / dimensions,
+                                                                                  generate_func,
+                                                                                  extra);
     }
 };
 
@@ -348,27 +363,26 @@ struct runner<curandStateScrambledSobol32_t>
     {
         this->dimensions = dimensions;
 
-        const size_t states_size = blocks * threads * dimensions;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&states),
-                             states_size * sizeof(curandStateScrambledSobol32_t)));
-
         curandDirectionVectors32_t* h_directions;
+        unsigned int*               h_constants;
+
         CURAND_CALL(
             curandGetDirectionVectors32(&h_directions, CURAND_DIRECTION_VECTORS_32_JOEKUO6));
+        CURAND_CALL(curandGetScrambleConstants32(&h_constants));
+
+        const size_t states_size = blocks * threads * dimensions;
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(curandStateScrambledSobol32_t)));
+
         unsigned int* directions;
-        const size_t  size = dimensions * sizeof(unsigned int) * 32;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&directions), size));
-        CUDA_CALL(cudaMemcpy(directions, h_directions, size, cudaMemcpyHostToDevice));
+        const size_t  directions_size = dimensions * sizeof(unsigned int) * 32;
+        CUDA_CALL(cudaMalloc(&directions, directions_size));
+        CUDA_CALL(cudaMemcpy(directions, h_directions, directions_size, cudaMemcpyHostToDevice));
 
-        unsigned int* h_scramble_constants;
-        CURAND_CALL(curandGetScrambleConstants32(&h_scramble_constants));
         unsigned int* scramble_constants;
         const size_t  constants_size = dimensions * sizeof(unsigned int);
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&scramble_constants), constants_size));
-        CUDA_CALL(cudaMemcpy(scramble_constants,
-                             h_scramble_constants,
-                             constants_size,
-                             cudaMemcpyHostToDevice));
+        CUDA_CALL(cudaMalloc(&scramble_constants, constants_size));
+        CUDA_CALL(
+            cudaMemcpy(scramble_constants, h_constants, constants_size, cudaMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         init_scrambled_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(
@@ -377,7 +391,7 @@ struct runner<curandStateScrambledSobol32_t>
             scramble_constants,
             static_cast<unsigned int>(offset));
 
-        CUDA_CALL(cudaPeekAtLastError());
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
 
         CUDA_CALL(cudaFree(directions));
@@ -397,17 +411,18 @@ struct runner<curandStateScrambledSobol32_t>
     template<typename T, typename GenerateFunc, typename Extra>
     void generate(const size_t        blocks,
                   const size_t        threads,
+                  cudaStream_t        stream,
                   T*                  data,
                   const size_t        size,
                   const GenerateFunc& generate_func,
                   const Extra         extra)
     {
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
-        generate_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(states,
-                                                                       data,
-                                                                       size / dimensions,
-                                                                       generate_func,
-                                                                       extra);
+        generate_sobol_kernel<<<dim3(blocks_x, dimensions), threads, 0, stream>>>(states,
+                                                                                  data,
+                                                                                  size / dimensions,
+                                                                                  generate_func,
+                                                                                  extra);
     }
 };
 
@@ -425,21 +440,22 @@ struct runner<curandStateSobol64_t>
     {
         this->dimensions = dimensions;
 
+        curandDirectionVectors64_t* h_directions;
+        CURAND_CALL(
+            curandGetDirectionVectors64(&h_directions, CURAND_DIRECTION_VECTORS_64_JOEKUO6));
+
         const size_t states_size = blocks * threads * dimensions;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&states),
-                             states_size * sizeof(curandStateSobol64_t)));
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(curandStateSobol64_t)));
 
-        curandDirectionVectors64_t * h_directions;
-        CURAND_CALL(curandGetDirectionVectors64(&h_directions, CURAND_DIRECTION_VECTORS_64_JOEKUO6));
         unsigned long long int* directions;
         const size_t            size = dimensions * sizeof(unsigned long long) * 64;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&directions), size));
+        CUDA_CALL(cudaMalloc(&directions, size));
         CUDA_CALL(cudaMemcpy(directions, h_directions, size, cudaMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         init_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(states, directions, offset);
 
-        CUDA_CALL(cudaPeekAtLastError());
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
 
         CUDA_CALL(cudaFree(directions));
@@ -458,6 +474,7 @@ struct runner<curandStateSobol64_t>
     template<typename T, typename GenerateFunc, typename Extra>
     void generate(const size_t        blocks,
                   const size_t        threads,
+                  cudaStream_t        stream,
                   T*                  data,
                   const size_t        size,
                   const GenerateFunc& generate_func,
@@ -486,27 +503,26 @@ struct runner<curandStateScrambledSobol64_t>
     {
         this->dimensions = dimensions;
 
-        const size_t states_size = blocks * threads * dimensions;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&states),
-                             states_size * sizeof(curandStateScrambledSobol64_t)));
-
         curandDirectionVectors64_t* h_directions;
+        unsigned long long*         h_constants;
+
         CURAND_CALL(
             curandGetDirectionVectors64(&h_directions, CURAND_DIRECTION_VECTORS_64_JOEKUO6));
+        CURAND_CALL(curandGetScrambleConstants64(&h_constants));
+
+        const size_t states_size = blocks * threads * dimensions;
+        CUDA_CALL(cudaMalloc(&states, states_size * sizeof(curandStateScrambledSobol64_t)));
+
         unsigned long long* directions;
-        const size_t        size = dimensions * sizeof(unsigned long long) * 64;
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&directions), size));
-        CUDA_CALL(cudaMemcpy(directions, h_directions, size, cudaMemcpyHostToDevice));
+        const size_t        directions_size = dimensions * sizeof(unsigned long long) * 64;
+        CUDA_CALL(cudaMalloc(&directions, directions_size));
+        CUDA_CALL(cudaMemcpy(directions, h_directions, directions_size, cudaMemcpyHostToDevice));
 
-        unsigned long long* h_scramble_constants;
-        CURAND_CALL(curandGetScrambleConstants64(&h_scramble_constants));
         unsigned long long* scramble_constants;
         const size_t        constants_size = dimensions * sizeof(unsigned long long);
-        CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&scramble_constants), constants_size));
-        CUDA_CALL(cudaMemcpy(scramble_constants,
-                             h_scramble_constants,
-                             constants_size,
-                             cudaMemcpyHostToDevice));
+        CUDA_CALL(cudaMalloc(&scramble_constants, constants_size));
+        CUDA_CALL(
+            cudaMemcpy(scramble_constants, h_constants, constants_size, cudaMemcpyHostToDevice));
 
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         init_scrambled_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(states,
@@ -514,7 +530,7 @@ struct runner<curandStateScrambledSobol64_t>
                                                                              scramble_constants,
                                                                              offset);
 
-        CUDA_CALL(cudaPeekAtLastError());
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
 
         CUDA_CALL(cudaFree(directions));
@@ -532,12 +548,13 @@ struct runner<curandStateScrambledSobol64_t>
     }
 
     template<typename T, typename GenerateFunc, typename Extra>
-    void generate(const size_t blocks,
-                  const size_t threads,
-                  T * data,
-                  const size_t size,
+    void generate(const size_t        blocks,
+                  const size_t        threads,
+                  cudaStream_t        stream,
+                  T*                  data,
+                  const size_t        size,
                   const GenerateFunc& generate_func,
-                  const Extra extra)
+                  const Extra         extra)
     {
         const size_t blocks_x = next_power2((blocks + dimensions - 1) / dimensions);
         generate_sobol_kernel<<<dim3(blocks_x, dimensions), threads>>>(states,
@@ -549,9 +566,13 @@ struct runner<curandStateScrambledSobol64_t>
 };
 
 template<typename T, typename GeneratorState, typename GenerateFunc, typename Extra>
-void run_benchmark(const cli::Parser& parser,
+void run_benchmark(const cli::Parser&  parser,
+                   cudaStream_t        stream,
                    const GenerateFunc& generate_func,
-                   const Extra extra)
+                   const Extra         extra,
+                   const std::string&  distribution,
+                   const std::string&  engine,
+                   const double        lambda = 0.f)
 {
     const size_t size = parser.get<size_t>("size");
     const size_t dimensions = parser.get<size_t>("dimensions");
@@ -560,137 +581,180 @@ void run_benchmark(const cli::Parser& parser,
     const size_t blocks = parser.get<size_t>("blocks");
     const size_t threads = parser.get<size_t>("threads");
 
+    const std::string format = parser.get<std::string>("format");
+
     T * data;
-    CUDA_CALL(cudaMalloc(reinterpret_cast<void**>(&data), size * sizeof(T)));
+    CUDA_CALL(cudaMalloc(&data, size * sizeof(T)));
 
     runner<GeneratorState> r(dimensions, blocks, threads, 12345ULL, 6789ULL);
 
     // Warm-up
     for (size_t i = 0; i < 5; i++)
     {
-        r.generate(blocks, threads, data, size, generate_func, extra);
-        CUDA_CALL(cudaPeekAtLastError());
+        r.generate(blocks, threads, stream, data, size, generate_func, extra);
+        CUDA_CALL(cudaGetLastError());
         CUDA_CALL(cudaDeviceSynchronize());
     }
     CUDA_CALL(cudaDeviceSynchronize());
 
     // Measurement
-    auto start = std::chrono::high_resolution_clock::now();
+    cudaEvent_t start, stop;
+    CUDA_CALL(cudaEventCreate(&start));
+    CUDA_CALL(cudaEventCreate(&stop));
+    CUDA_CALL(cudaEventRecord(start, stream));
     for (size_t i = 0; i < trials; i++)
     {
-        r.generate(blocks, threads, data, size, generate_func, extra);
+        r.generate(blocks, threads, stream, data, size, generate_func, extra);
     }
-    CUDA_CALL(cudaPeekAtLastError());
-    CUDA_CALL(cudaDeviceSynchronize());
-    auto end = std::chrono::high_resolution_clock::now();
-    std::chrono::duration<double, std::milli> elapsed = end - start;
+    CUDA_CALL(cudaEventRecord(stop, stream));
+    CUDA_CALL(cudaEventSynchronize(stop));
+    float elapsed;
+    CUDA_CALL(cudaEventElapsedTime(&elapsed, start, stop));
+    CUDA_CALL(cudaEventDestroy(start));
+    CUDA_CALL(cudaEventDestroy(stop));
 
-    std::cout << std::fixed << std::setprecision(3)
-              << "      "
-              << "Throughput = "
-              << std::setw(8) << (trials * size * sizeof(T)) /
-                    (elapsed.count() / 1e3 * (1 << 30))
-              << " GB/s, Samples = "
-              << std::setw(8) << (trials * size) /
-                    (elapsed.count() / 1e3 * (1 << 30))
-              << " GSample/s, AvgTime (1 trial) = "
-              << std::setw(8) << elapsed.count() / trials
-              << " ms, Time (all) = "
-              << std::setw(8) << elapsed.count()
-              << " ms, Size = " << size
-              << std::endl;
+    if(format.compare("csv") == 0)
+    {
+        std::cout << std::fixed << std::setprecision(3) << engine << "," << distribution << ","
+                  << (trials * size * sizeof(T)) / (elapsed / 1e3 * (1 << 30)) << ","
+                  << (trials * size) / (elapsed / 1e3 * (1 << 30)) << "," << elapsed / trials << ","
+                  << elapsed << "," << size << ",";
+        if(distribution.compare("poisson") == 0 || distribution.compare("discrete-poisson") == 0)
+        {
+            std::cout << lambda;
+        }
+        std::cout << std::endl;
+    }
+    else
+    {
+        if(format.compare("console") != 0)
+        {
+            std::cout << "Unknown format specified (must be either console or csv).  Defaulting to "
+                         "console output."
+                      << std::endl;
+        }
+        std::cout << std::fixed << std::setprecision(3) << "      "
+                  << "Throughput = " << std::setw(8)
+                  << (trials * size * sizeof(T)) / (elapsed / 1e3 * (1 << 30))
+                  << " GB/s, Samples = " << std::setw(8)
+                  << (trials * size) / (elapsed / 1e3 * (1 << 30))
+                  << " GSample/s, AvgTime (1 trial) = " << std::setw(8) << elapsed / trials
+                  << " ms, Time (all) = " << std::setw(8) << elapsed << " ms, Size = " << size
+                  << std::endl;
+    }
 
     CUDA_CALL(cudaFree(data));
 }
 
 template<typename GeneratorState>
 void run_benchmarks(const cli::Parser& parser,
-                    const std::string& distribution)
+                    const std::string& distribution,
+                    const std::string& engine,
+                    cudaStream_t       stream)
 {
+    const std::string format = parser.get<std::string>("format");
     if (distribution == "uniform-uint")
     {
-        if (!std::is_same<GeneratorState, curandStateSobol64_t>::value &&
-            !std::is_same<GeneratorState, curandStateScrambledSobol64_t>::value)
-        {
-            run_benchmark<unsigned int, GeneratorState>(parser,
-                [] __device__ (GeneratorState * state, int) {
-                    return curand(state);
-                }, 0
-            );
-        }
+        run_benchmark<unsigned int, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand(state); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "uniform-long-long")
     {
         if (std::is_same<GeneratorState, curandStateSobol64_t>::value ||
             std::is_same<GeneratorState, curandStateScrambledSobol64_t>::value)
         {
-            run_benchmark<unsigned long long, GeneratorState>(parser,
-                [] __device__ (GeneratorState * state, int) {
-                    return curand(state);
-                }, 0
-            );
+            run_benchmark<unsigned long long, GeneratorState>(
+                parser,
+                stream,
+                [] __device__ ( GeneratorState* state, int) { return curand(state); },
+                0,
+                distribution,
+                engine);
         }
     }
     if (distribution == "uniform-float")
     {
-        run_benchmark<float, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_uniform(state);
-            }, 0
-        );
+        run_benchmark<float, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_uniform(state); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "uniform-double")
     {
-        run_benchmark<double, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_uniform_double(state);
-            }, 0
-        );
+        run_benchmark<double, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_uniform_double(state); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "normal-float")
     {
-        run_benchmark<float, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_normal(state);
-            }, 0
-        );
+        run_benchmark<float, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_normal(state); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "normal-double")
     {
-        run_benchmark<double, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_normal_double(state);
-            }, 0
-        );
+        run_benchmark<double, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_normal_double(state); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "log-normal-float")
     {
-        run_benchmark<float, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_log_normal(state, 0.0f, 1.0f);
-            }, 0
-        );
+        run_benchmark<float, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_log_normal(state, 0.0f, 1.0f); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "log-normal-double")
     {
-        run_benchmark<double, GeneratorState>(parser,
-            [] __device__ (GeneratorState * state, int) {
-                return curand_log_normal_double(state, 0.0, 1.0);
-            }, 0
-        );
+        run_benchmark<double, GeneratorState>(
+            parser,
+            stream,
+            [] __device__ ( GeneratorState* state, int) { return curand_log_normal_double(state, 0.0, 1.0); },
+            0,
+            distribution,
+            engine);
     }
     if (distribution == "poisson")
     {
         const auto lambdas = parser.get<std::vector<double>>("lambda");
         for (double lambda : lambdas)
         {
-            std::cout << "    " << "lambda "
-                 << std::fixed << std::setprecision(1) << lambda << std::endl;
-            run_benchmark<unsigned int, GeneratorState>(parser,
-                [] __device__ (GeneratorState * state, double lambda) {
-                    return curand_poisson(state, lambda);
-                }, lambda
-            );
+            if(format.compare("console") == 0)
+            {
+                std::cout << "    "
+                          << "lambda " << std::fixed << std::setprecision(1) << lambda << std::endl;
+            }
+            run_benchmark<unsigned int, GeneratorState>(
+                parser,
+                stream,
+                [] __device__ ( GeneratorState* state, double lambda) { return curand_poisson(state, lambda); },
+                lambda,
+                distribution,
+                engine,
+                lambda);
         }
     }
     if (distribution == "discrete-poisson")
@@ -698,15 +762,22 @@ void run_benchmarks(const cli::Parser& parser,
         const auto lambdas = parser.get<std::vector<double>>("lambda");
         for (double lambda : lambdas)
         {
-            std::cout << "    " << "lambda "
-                 << std::fixed << std::setprecision(1) << lambda << std::endl;
+            if(format.compare("console") == 0)
+            {
+                std::cout << "    "
+                          << "lambda " << std::fixed << std::setprecision(1) << lambda << std::endl;
+            }
             curandDiscreteDistribution_t discrete_distribution;
             CURAND_CALL(curandCreatePoissonDistribution(lambda, &discrete_distribution));
-            run_benchmark<unsigned int, GeneratorState>(parser,
-                [] __device__ (GeneratorState * state, curandDiscreteDistribution_t discrete_distribution) {
-                    return curand_discrete(state, discrete_distribution);
-                }, discrete_distribution
-            );
+            run_benchmark<unsigned int, GeneratorState>(
+                parser,
+                stream,
+                [] __device__ ( GeneratorState* state, curandDiscreteDistribution_t discrete_distribution)
+                { return curand_discrete(state, discrete_distribution); },
+                discrete_distribution,
+                distribution,
+                engine,
+                lambda);
             CURAND_CALL(curandDestroyDistribution(discrete_distribution));
         }
     }
@@ -764,6 +835,10 @@ int main(int argc, char *argv[])
     parser.set_optional<std::vector<std::string>>("dis", "dis", {"uniform-uint"}, distribution_desc.c_str());
     parser.set_optional<std::vector<std::string>>("engine", "engine", {"philox"}, engine_desc.c_str());
     parser.set_optional<std::vector<double>>("lambda", "lambda", {10.0}, "space-separated list of lambdas of Poisson distribution");
+    parser.set_optional<std::string>("format",
+                                     "format",
+                                     {"console"},
+                                     "output format: console or csv");
     parser.run_and_exit_if_error();
 
     std::vector<std::string> engines;
@@ -809,53 +884,75 @@ int main(int argc, char *argv[])
     cudaDeviceProp props;
     CUDA_CALL(cudaGetDeviceProperties(&props, device_id));
 
+    std::cout << "benchmark_curand_kernel" << std::endl;
     std::cout << "cuRAND: " << version << " ";
     std::cout << "Runtime: " << runtime_version << " ";
     std::cout << "Device: " << props.name;
     std::cout << std::endl << std::endl;
 
+    cudaStream_t stream;
+    CUDA_CALL(cudaStreamCreate(&stream));
+
+    std::string format         = parser.get<std::string>("format");
+    bool        console_output = format.compare("console") == 0 ? true : false;
+
+    if(!console_output)
+    {
+        std::cout
+            << "Engine,Distribution,Throughput,Samples,AvgTime (1 Trial),Time(all),Size,Lambda"
+            << std::endl;
+        std::cout << ",,GB/s,GSample/s,ms),ms),values," << std::endl;
+    }
     for (auto engine : engines)
     {
-        std::cout << engine << ":" << std::endl;
+        if(console_output)
+        {
+            std::cout << engine << ":" << std::endl;
+        }
         for (auto distribution : distributions)
         {
-            std::cout << "  " << distribution << ":" << std::endl;
+            if(console_output)
+            {
+                std::cout << engine << ":" << std::endl;
+            }
             const std::string plot_name = engine + "-" + distribution;
             if (engine == "xorwow")
             {
-                run_benchmarks<curandStateXORWOW_t>(parser, distribution);
+                run_benchmarks<curandStateXORWOW_t>(parser, distribution, engine, stream);
             }
             else if (engine == "mrg32k3a")
             {
-                run_benchmarks<curandStateMRG32k3a_t>(parser, distribution);
+                run_benchmarks<curandStateMRG32k3a_t>(parser, distribution, engine, stream);
             }
             else if (engine == "philox")
             {
-                run_benchmarks<curandStatePhilox4_32_10_t>(parser, distribution);
+                run_benchmarks<curandStatePhilox4_32_10_t>(parser, distribution, engine, stream);
             }
             else if (engine == "sobol32")
             {
-                run_benchmarks<curandStateSobol32_t>(parser, distribution);
+                run_benchmarks<curandStateSobol32_t>(parser, distribution, engine, stream);
             }
             else if(engine == "scrambled_sobol32")
             {
-                run_benchmarks<curandStateScrambledSobol32_t>(parser, distribution);
+                run_benchmarks<curandStateScrambledSobol32_t>(parser, distribution, engine, stream);
             }
             else if (engine == "sobol64")
             {
-                run_benchmarks<curandStateSobol64_t>(parser, distribution);
+                run_benchmarks<curandStateSobol64_t>(parser, distribution, engine, stream);
             }
             else if(engine == "scrambled_sobol64")
             {
-                run_benchmarks<curandStateScrambledSobol64_t>(parser, distribution);
+                run_benchmarks<curandStateScrambledSobol64_t>(parser, distribution, engine, stream);
             }
             else if (engine == "mtgp32")
             {
-                run_benchmarks<curandStateMtgp32_t>(parser, distribution);
+                run_benchmarks<curandStateMtgp32_t>(parser, distribution, engine, stream);
             }
         }
         std::cout << std::endl;
     }
 
+    CUDA_CALL(cudaStreamDestroy(stream));
+
     return 0;
 }
