@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -155,17 +155,11 @@ inline void customCSVReporter::PrintRunData(const Run& run)
     std::ostream& Err = GetErrorStream();
 
     //get the name of the engine and distribution:
-
     std::string temp = run.benchmark_name();
-
     std::string deviceName = std::string(temp.begin(), temp.begin() + temp.find("<"));
-
     temp.erase(0, temp.find("<") + 1);
-
     std::string engineName = std::string(temp.begin(), temp.begin() + temp.find(","));
-
     temp.erase(0, engineName.size() + 1);
-
     std::string mode = "default";
 
     if(deviceName != "device_kernel")
@@ -174,7 +168,6 @@ inline void customCSVReporter::PrintRunData(const Run& run)
         temp.erase(0, temp.find(",") + 1);
     }
     std::string disName = std::string(temp.begin(), temp.begin() + temp.find(">"));
-
     std::string lambda = "";
 
     size_t ePos = disName.find("=");
@@ -186,11 +179,11 @@ inline void customCSVReporter::PrintRunData(const Run& run)
 
     Out << engineName << "," << disName << "," << mode << ",";
     Out << CsvEscape(run.benchmark_name()) << ",";
-    if(run.error_occurred)
+    if(run.skipped)
     {
         Err << std::string(elements.size() - 3, ',');
         Err << "true,";
-        Err << CsvEscape(run.error_message) << "\n";
+        Err << CsvEscape(run.skip_message) << "\n";
         return;
     }
 
