@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,14 +21,12 @@
 #ifndef ROCRAND_RNG_DISTRIBUTION_NORMAL_H_
 #define ROCRAND_RNG_DISTRIBUTION_NORMAL_H_
 
-#include "../common.hpp"
+#include "../common.hpp" // IWYU pragma: keep
 
 #include <rocrand/rocrand.h>
 #include <rocrand/rocrand_normal.h>
 
-#include <hip/hip_runtime.h>
-
-#include <math.h>
+#include <hip/hip_runtime_api.h>
 
 namespace rocrand_impl::host
 {
