@@ -1,4 +1,4 @@
-// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -34,10 +34,11 @@
 #include "test_rocrand_common.hpp"
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_init_kernel(GeneratorState*    states,
-                                                          const size_t       states_size,
-                                                          unsigned long long seed,
-                                                          unsigned long long offset)
+__global__
+void rocrand_init_kernel(GeneratorState*    states,
+                         const size_t       states_size,
+                         unsigned long long seed,
+                         unsigned long long offset)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int subsequence = state_id;
@@ -50,7 +51,8 @@ __global__ __launch_bounds__(32) void rocrand_init_kernel(GeneratorState*    sta
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_kernel(unsigned long long* output, const size_t size)
+__global__
+void rocrand_kernel(unsigned int* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -68,7 +70,8 @@ __global__ __launch_bounds__(32) void rocrand_kernel(unsigned long long* output,
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_uniform_kernel(float* output, const size_t size)
+__global__
+void rocrand_uniform_kernel(float* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -86,8 +89,8 @@ __global__ __launch_bounds__(32) void rocrand_uniform_kernel(float* output, cons
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_uniform_double_kernel(double*      output,
-                                                                    const size_t size)
+__global__
+void rocrand_uniform_double_kernel(double* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -105,7 +108,8 @@ __global__ __launch_bounds__(32) void rocrand_uniform_double_kernel(double*
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_normal_kernel(float* output, const size_t size)
+__global__
+void rocrand_normal_kernel(float* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -126,7 +130,8 @@ __global__ __launch_bounds__(32) void rocrand_normal_kernel(float* output, const
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_log_normal_kernel(float* output, const size_t size)
+__global__
+void rocrand_log_normal_kernel(float* output, const size_t size)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -147,9 +152,8 @@ __global__ __launch_bounds__(32) void rocrand_log_normal_kernel(float* output, c
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(64) void rocrand_poisson_kernel(unsigned int* output,
-                                                             const size_t  size,
-                                                             double        lambda)
+__global__
+void rocrand_poisson_kernel(unsigned int* output, const size_t size, double lambda)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -167,8 +171,10 @@ __global__ __launch_bounds__(64) void rocrand_poisson_kernel(unsigned int* outpu
 }
 
 template<class GeneratorState>
-__global__ __launch_bounds__(32) void rocrand_discrete_kernel(
-    unsigned int* output, const size_t size, rocrand_discrete_distribution discrete_distribution)
+__global__
+void rocrand_discrete_kernel(unsigned int*                 output,
+                             const size_t                  size,
+                             rocrand_discrete_distribution discrete_distribution)
 {
     const unsigned int state_id    = blockIdx.x * blockDim.x + threadIdx.x;
     const unsigned int global_size = gridDim.x * blockDim.x;
@@ -185,23 +191,23 @@ __global__ __launch_bounds__(32) void rocrand_discrete_kernel(
     }
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_state_threefry_type)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_state_threefry_type)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
-    EXPECT_EQ(alignof(state_type), alignof(ulonglong2));
+    typedef rocrand_state_threefry2x32_20 state_type;
+    EXPECT_EQ(alignof(state_type), alignof(uint2));
     EXPECT_EQ(sizeof(state_type[32]), 32 * sizeof(state_type));
-    // TODO: Enable once ulonglong2 trivially copyable.
+    // TODO: Enable once uint2 trivially copyable.
     //EXPECT_TRUE(std::is_trivially_copyable<state_type>::value);
     EXPECT_TRUE(std::is_trivially_destructible<state_type>::value);
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand)
+TEST(rocrand_kernel_threefry2x32_20, rocrand)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
-    const size_t        output_size = 8192;
-    unsigned long long* output;
-    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned long long)));
+    const size_t  output_size = 8192;
+    unsigned int* output;
+    HIP_CHECK(hipMallocHelper(&output, output_size * sizeof(unsigned int)));
     HIP_CHECK(hipDeviceSynchronize());
 
     hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_kernel<state_type>),
@@ -213,10 +219,10 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand)
                        output_size);
     HIP_CHECK(hipGetLastError());
 
-    std::vector<unsigned long long> output_host(output_size);
+    std::vector<unsigned int> output_host(output_size);
     HIP_CHECK(hipMemcpy(output_host.data(),
                         output,
-                        output_size * sizeof(unsigned long long),
+                        output_size * sizeof(unsigned int),
                         hipMemcpyDeviceToHost));
     HIP_CHECK(hipDeviceSynchronize());
     HIP_CHECK(hipFree(output));
@@ -224,15 +230,15 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand)
     double mean = 0;
     for(auto v : output_host)
     {
-        mean += static_cast<double>(v) / static_cast<double>(ULLONG_MAX);
+        mean += static_cast<double>(v) / UINT_MAX;
     }
     mean = mean / output_size;
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -263,9 +269,9 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 8192;
     double*      output;
@@ -296,9 +302,9 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double)
     EXPECT_NEAR(mean, 0.5, 0.1);
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_range)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_range)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 1 << 26;
     float*       output;
@@ -327,9 +333,9 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_range)
     }
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double_range)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_uniform_double_range)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 1 << 26;
     double*      output;
@@ -358,9 +364,9 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_uniform_double_range)
     }
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_normal)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_normal)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -399,9 +405,9 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_normal)
     EXPECT_NEAR(stddev, 1.0, 0.2);
 }
 
-TEST(rocrand_kernel_threefry2x64_20, rocrand_log_normal)
+TEST(rocrand_kernel_threefry2x32_20, rocrand_log_normal)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const size_t output_size = 8192;
     float*       output;
@@ -444,12 +450,12 @@ TEST(rocrand_kernel_threefry2x64_20, rocrand_log_normal)
     EXPECT_NEAR(0.25, logstd, 0.25 * 0.2);
 }
 
-class rocrand_kernel_threefry2x64_20_poisson : public ::testing::TestWithParam<double>
+class rocrand_kernel_threefry2x32_20_poisson : public ::testing::TestWithParam<double>
 {};
 
-TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_poisson)
+TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_poisson)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const double lambda = GetParam();
 
@@ -494,9 +500,9 @@ TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_poisson)
     EXPECT_NEAR(variance, lambda, std::max(1.0, lambda * 1e-1));
 }
 
-TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_discrete)
+TEST_P(rocrand_kernel_threefry2x32_20_poisson, rocrand_discrete)
 {
-    typedef rocrand_state_threefry2x64_20 state_type;
+    typedef rocrand_state_threefry2x32_20 state_type;
 
     const double lambda = GetParam();
 
@@ -508,14 +514,14 @@ TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_discrete)
     rocrand_discrete_distribution discrete_distribution;
     ROCRAND_CHECK(rocrand_create_poisson_distribution(lambda, &discrete_distribution));
 
-    hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_poisson_kernel<state_type>),
+    hipLaunchKernelGGL(HIP_KERNEL_NAME(rocrand_discrete_kernel<state_type>),
                        dim3(4),
                        dim3(64),
                        0,
                        0,
                        output,
                        output_size,
-                       lambda);
+                       discrete_distribution);
     HIP_CHECK(hipGetLastError());
 
     std::vector<unsigned int> output_host(output_size);
@@ -547,6 +553,6 @@ TEST_P(rocrand_kernel_threefry2x64_20_poisson, rocrand_discrete)
 
 const double lambdas[] = {1.0, 5.5, 20.0, 100.0, 1234.5, 5000.0};
 
-INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry2x64_20_poisson,
-                         rocrand_kernel_threefry2x64_20_poisson,
+INSTANTIATE_TEST_SUITE_P(rocrand_kernel_threefry2x32_20_poisson,
+                         rocrand_kernel_threefry2x32_20_poisson,
                          ::testing::ValuesIn(lambdas));
